precision highp float;
uniform sampler2D origInputTexture;
uniform sampler2D pass0Texture;
varying vec2 textureCoordinate;

uniform vec2 key_position;
uniform vec3 key_color;

uniform int mode; // 0 - sample key color from key_position, otherwise use uniform key_color

uniform float intensity;

float desaturate(vec3 p) {
    return dot(p, vec3(0.299, 0.587, 0.114));
}

void main() {
    vec4 texColor = texture2D(pass0Texture, textureCoordinate);

    // sample key color from image or take the value from uniform
    vec3 key = mode == 0 ? texture2D(origInputTexture, key_position).rgb : key_color;
    
    vec3 norm_key = normalize(key);

    float likelihood = dot(normalize(texColor.rgb), norm_key);

    if (likelihood > 0.9) {
        float shadow_amount = 1.0 - length(dot(texColor.rgb, norm_key)) / length(key);
        shadow_amount = clamp(shadow_amount, 0.0, 1.0);

        texColor.a *= step(intensity, shadow_amount) * smoothstep(intensity * 0.9, intensity, shadow_amount);

        float desat = desaturate(texColor.rgb);

        // desaturate the image color based on alpha
        texColor.rgb = mix(vec3(desat), texColor.rgb, texColor.a);
    }

    gl_FragColor = texColor;
}