precision highp float;
uniform sampler2D inputTexture;
varying vec2 textureCoordinate;
varying vec2 maskTextureCoordinate;
uniform sampler2D u_mask;

uniform float u_diff;
uniform float u_invert;
uniform vec2 u_size;

bool inMask(vec2 uv, float sizeMultiplier) {
    uv /= u_size * sizeMultiplier;
    uv += 0.5;
    if (uv != clamp(uv, 0.0, 1.0)) {
        return false;
    }
    return texture2D(u_mask, uv).rgb != vec3(0);
}

#define BINARY_SEARCH_PRECISION 20

float getFadeDistance(vec2 uv) {
    float diff = max(u_diff, 1e-5);
    float lt = max(0.0, 1.0 - diff), rt = 1.0 + diff;

    float mid;
    for (int i = 0; i < BINARY_SEARCH_PRECISION; ++i) {
        mid = (lt + rt) * 0.5;
        if (inMask(uv, mid)) {
            rt = mid;
        } else {
            lt = mid;
        }
    }

    // 在轮廓内部小于-0.5~0，外部0~0.5
    return (mid - (1.0 - diff)) / (2.0 * diff) - 0.5;
}

void main() {
    float fade = smoothstep(0.0, 0.5, getFadeDistance(maskTextureCoordinate));
    gl_FragColor = vec4(u_invert > 0.0 ? fade : 1.0 - fade) * texture2D(inputTexture, textureCoordinate);
    //gl_FragColor = vec4(vec3(1.0, 1.0, 1.0), u_invert > 0.0 ? fade : 1.0 - fade) * texture2D(inputTexture, textureCoordinate);
}
