precision highp float;
uniform sampler2D inputTexture;
varying vec2 textureCoordinate;

uniform float u_diff;
uniform float u_invert;
uniform float u_rotation;
uniform vec2 u_center;
uniform vec2 u_rect_size;
uniform vec2 u_input_size;
uniform float u_corner_radius;

void main() {
    vec2 coord = (textureCoordinate - u_center) * u_input_size;
    coord = mat2(cos(u_rotation), sin(u_rotation),
                 -sin(u_rotation), cos(u_rotation)) * coord;

    vec2 half_size = u_rect_size * 0.5;

    bvec2 in_cross = lessThan(abs(coord), half_size - u_corner_radius);

    float value;

    if (all(in_cross)) {
        value = 0.0;
    } else  {
        vec2 min_dist = abs(coord) - half_size;
        value = max(min_dist.x, min_dist.y);
    }

    float diff = max(u_diff, 1e-5);
    float fade = value >= 0.0 ? smoothstep(0.0, 1.0, clamp(value, 0.0, diff) / diff) : 0.0;

    gl_FragColor = vec4(vec3(1.0, 1.0, 1.0), u_invert > 0.0 ? fade : 1.0 - fade) * texture2D(inputTexture, textureCoordinate);
}
